import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:syncfusion_flutter_charts/charts.dart';
import 'package:syncfusion_flutter_charts/sparkcharts.dart';

void main() {
  return runApp(_ChartApp());
}

class _ChartApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      theme: ThemeData(primarySwatch: Colors.blue),
      home: _MyHomePage(),
    );
  }
}

class _MyHomePage extends StatefulWidget {
  // ignore: prefer_const_constructors_in_immutables
  _MyHomePage({Key? key}) : super(key: key);

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<_MyHomePage> {
  List<_SalesData> data = [
    _SalesData(0, 35),
    _SalesData(1, 28),
    _SalesData(2, 34),
    _SalesData(4, 32),
    _SalesData(5, 40)
  ];
  //Initialize the series controller
  ChartSeriesController? _chartSeriesController;
  double minX = 0, maxX = 0, minY = 0;

  @override
  Widget build(BuildContext context) {
    SchedulerBinding.instance.addPostFrameCallback(
      (_) {
        // Stored the origin point of the chart to get the pixel value.
        CartesianChartPoint<dynamic> chartPoint1 =
            CartesianChartPoint<dynamic>(minX, minY);
        // Stored the maximum x-point value point of the chart to get the pixel value.
        CartesianChartPoint<dynamic> chartPoint2 =
            CartesianChartPoint<dynamic>(maxX, minY);
        // Calcualted the pixel values of the points
        Offset point1 = _chartSeriesController!.pointToPixel(chartPoint1);
        Offset point2 = _chartSeriesController!.pointToPixel(chartPoint2);
        // once the pixel values are calcualted. Deduct second point's x pixel value with first point's x pixel value
        // to get the width of the chart area
        print('Chart area width: ${(point2.dx - point1.dx).toString()}');
        // Also to note that, if you are using the point1 and point2 values to draw anything on the chart area,
        // please add their x pixel offset values with default margin value of the chart
        // (which is 10 by default, because, 5 pixel margin added on both sides) {point2.dx + 10}, {point1.dx + 10}
      },
    );
    return Scaffold(
        appBar: AppBar(
          title: const Text('Syncfusion Flutter chart'),
        ),
        body: Column(children: [
          //Initialize the chart widget
          SfCartesianChart(
              // margin: EdgeInsets.all(0),
              primaryXAxis: NumericAxis(),
              onActualRangeChanged: (args) {
                if (args.axisName == 'primaryXAxis') {
                  // Retrieved the visible minimum and maximum value of x-axis
                  minX = args.visibleMin.toDouble();
                  maxX = args.visibleMax.toDouble();
                } else {
                  // Retrieved the visible minimum value of y-axis
                  minY = args.visibleMin.toDouble();
                }
              },
              series: <ChartSeries<_SalesData, num>>[
                ColumnSeries<_SalesData, num>(
                    dataSource: data,
                    xValueMapper: (_SalesData sales, _) => sales.year,
                    yValueMapper: (_SalesData sales, _) => sales.sales,
                    name: 'Sales',
                    //Initialize the onRendererCreated event and store the controller for the respective series
                    onRendererCreated: (ChartSeriesController controller) {
                      _chartSeriesController = controller;
                    })
              ]),
        ]));
  }
}

class _SalesData {
  _SalesData(this.year, this.sales);

  final double year;
  final double sales;
}
